<?php

/*
Plugin Name: Root Relative URLs
Plugin URI: http://www.marcuspope.com/wordpress/
Description: A Wordpress Plugin that converts all URL formats to root relative URL's to enable seamless transitioning
between staging and production host environments without requiring hackish content find-replace or host/NAT
spoofing strategies.
Author: Marcus E. Pope, marcuspope
Author URI: http://www.marcuspope.com
Version: 1.0

Copyright 2011 Marcus E. Pope (email : me@marcuspope.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

//Ideally this function runs before any other plugin
add_action('plugins_loaded', array('MP_WP_Root_Relative_URLS', 'init'), 1);

class MP_WP_Root_Relative_URLS {
    //Wordpress Root Relative URL Hack - this allows for accessing a Wordpress site from multiple domain names and
    //munges absolute urls provided to the wysiwyg editor to be root relative instead of absolute
    //generally displays urls throughout the admin area as root relative instead of absolute
    static function add_actions($tag_arry, $func, $p = 10, $a = 1) {
        //allows for multiple tags to bind to the same funciton call
        //useful when using lambda functions
        foreach ($tag_arry as $v) {
            add_filter($v, $func, $p, $a);
        }
    }

    static function scheme( $url ) {
        //And this here is a prime example of why absolute urls in Wordpress create extra overhead and processing.
        //And in the core, they use four different approaches to acheive this translation!
        //For reference, see: http://core.trac.wordpress.org/ticket/19037
        if (is_ssl()) {
            $url = str_replace('http://', 'https://', $url);
        }
        else {
            $url = str_replace('https://', 'http://', $url);
        }
        return $url;
    }

    static function dynamic_absolute_url($url) {
        //These URL's cannot be reformmated into root-relative urls for various reasons.  Either they are indescriminantly
        //used in export functions or RSS feeds.  Or because they are literally checked against the domain of URLs stored
        //in the database - a needless process for any website.
        $url = parse_url($url);
        $relative = ltrim(@$url['path'], '/') . (isset($url['query']) ? "?" . $url['query'] : '');
        return MP_WP_Root_Relative_URLS::scheme( 'http://' . $_SERVER['HTTP_HOST'] . (!empty($relative) ? '/' . $relative: '') );
    }

    static function dynamic_rss_absolute_url($info, $type = 'url') {
        //Generates dynamic absolute URI's for RSS Feeds
        //get_bloginfo_rss returns multiple types of info, the_permalink_rss only returns a url.
        //so when type is not passed in, consider it a url, otherwise only parse when type == url
        if ($type == 'url') {
            return MP_WP_Root_Relative_URLS::dynamic_absolute_url($info);
        }
        return $info;
    }

    static function proper_root_relative_url($url) {
        //This method is used for urls that can be acceptably reformatted into root-relative urls without causing issues
        //related to other deficiencies in the wp core.
        $url = parse_url($url);
        return '/' . ltrim(@$url['path'], '/') . (isset($url['query']) ? "?" . $url['query'] : '');
    }

    static function proper_multisite_path_comparison($redirect_bool) {
        //Prevents infinite loop caused by the path matching but not the domain when making network admin requests
        global $current_blog, $current_site;

        //don't worry about domain name mismatch as long as the paths are correct
        if ($redirect_bool &&
            $current_blog->path == $current_site->path) {
            $redirect_bool = false;
        }

        return $redirect_bool;
    }

    static function root_relative_image_urls($html, $id, $caption, $title, $align, $url, $size, $alt) {
        //Same as media_send_to_editor but images are handled separately
        $p = parse_url($url);
        $root = $p['scheme'] . "://" . $p['host'];
        $html = str_ireplace($root, '', $html);
        return $html;
    }

    static function root_relative_media_urls($html, $id, $att) {
        //Filter out host from embed urls
        $p = parse_url($att['url']);
        $root = $p['scheme'] . "://" . $p['host'];
        $html = str_ireplace($root, '', $html);
        return $html;
    }

    static function init() {
        //Setup all hooks / filters for either dynamically replacing the host part of a URL with the current host
        //or for stripping the scheme + host + port altogether
        MP_WP_Root_Relative_URLS::add_actions(
            array(
                'option_siteurl',
                'blog_option_siteurl',
                'option_home',
                'admin_url',
                'home_url',
                'includes_url',
                'site_url',
                'site_option_siteurl',
                'network_home_url',
                'network_site_url'
            ),
            array(
                'MP_WP_Root_Relative_URLS',
                'dynamic_absolute_url'
            ),
            1
        );

        MP_WP_Root_Relative_URLS::add_actions(
            array(
                'post_link',
                'page_link',
                'attachment_link',
                'post_type_link',
                'wp_get_attachment_url'
            ),
            array(
                'MP_WP_Root_Relative_URLS',
                'proper_root_relative_url'
            ),
            1
        );

        MP_WP_Root_Relative_URLS::add_actions(
            array(
                'get_bloginfo_rss',
                'the_permalink_rss'
            ),
            array(
                'MP_WP_Root_Relative_URLS',
                'dynamic_rss_absolute_url'
            ),
            1, //high priority
            2  //supply second parameter for type checking
        );

        //HACK: This plugin actually won't work for MU Sites until either of the following conditions are true:
            //1. Wordpress core team publishes this patch - http://core.trac.wordpress.org/attachment/ticket/18910/ms-blogs.php.patch
            //2. You deal with the consequences of patching a core file yourself using the above patch reference
        //Regardless of the above, this plugin only supports path-based MU installations - at this point domain-based MU installations are not supported
        add_filter(
            'redirect_network_admin_request',
            array(
                'MP_WP_Root_Relative_URLS',
                'proper_multisite_path_comparison'
            )
        );

        add_filter(
            'image_send_to_editor',
            array(
                'MP_WP_Root_Relative_URLS',
                'root_relative_image_urls'
            ),
            1, //high priority
            8  //eight params? wow
        );

        add_filter(
            'media_send_to_editor',
            array(
                'MP_WP_Root_Relative_URLS',
                'root_relative_media_urls'
            ),
            1,
            3
        );
    }
}